package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Collections;
import java.util.Map;

import co.hyperverge.hypersnapsdk.activities.HVDocsActivity;
import co.hyperverge.hypersnapsdk.listeners.DocCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVDocConfig;


public class FLHVDocsCapture {

    private static final String TAG = FLHVDocsCapture.class.getSimpleName();

    private HVDocConfig docConfig;

    public void createNewDocConfig() {
        this.docConfig = new HVDocConfig();
    }

    private HVDocConfig getDocConfig() {
        if (this.docConfig == null) {
            this.docConfig = new HVDocConfig();
        }
        return this.docConfig;
    }

    @SuppressWarnings("unchecked")
    public void setHVDocConfig(Map<String, Object> hvDocConfig) {
        Map<String, Object> docCaptureUIStrings = (Map<String, Object>) hvDocConfig.get(FLHyperSnapConstants.DOC_CAPTURE_UI_STRINGS);
        if (docCaptureUIStrings != null && !docCaptureUIStrings.isEmpty()) {
            setUIStrings(docCaptureUIStrings);
        }

        Boolean shouldShowTrustLogos = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_TRUST_LOGOS);
        if (shouldShowTrustLogos != null) {
            getDocConfig().setShowTrustLogos(shouldShowTrustLogos);
        }

        Boolean shouldShowInstructionPage = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_INSTRUCTION_PAGE);
        if (shouldShowInstructionPage != null) {
            getDocConfig().setShouldShowInstructionPage(shouldShowInstructionPage);
        }

        Boolean shouldEnableDocUpload = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_ENABLE_UPLOAD);
        if (shouldEnableDocUpload != null) {
            getDocConfig().setEnableDocumentUpload(shouldEnableDocUpload);
        }

        Boolean shouldShowFlashIcon = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_FLASH_ICON);
        if (shouldShowFlashIcon != null) {
            getDocConfig().setShouldShowFlashIcon(shouldShowFlashIcon);
        }

        String docTypeValue = (String) hvDocConfig.get(FLHyperSnapConstants.DOC_TYPE_VALUE);
        Double aspectRatioValue = (Double) hvDocConfig.get(FLHyperSnapConstants.DOC_ASPECT_RATIO_VALUE);
        if (docTypeValue != null) {
            try {
                HVDocConfig.Document doc = HVDocConfig.Document.valueOf(docTypeValue);
                if (aspectRatioValue != null) doc.setAspectRatio(aspectRatioValue.floatValue());
                getDocConfig().setDocumentType(doc);
            } catch (Exception exp) {
                Log.e(TAG, exp.getMessage());
            }
        }

        Boolean shouldSetPadding = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SET_PADDING);
        if (shouldSetPadding != null) {
            getDocConfig().setShouldAddPadding(shouldSetPadding);
        }

        Double padding = (Double) hvDocConfig.get(FLHyperSnapConstants.DOC_DEFAULT_PADDING_VALUE);
        if (padding != null) {
            getDocConfig().setPadding(padding.floatValue());
        }

        Boolean shouldShowCloseAlert = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_CLOSE_ALERT);
        String closeAlertDialogTitle = (String) hvDocConfig.get(FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_TITLE);
        String closeAlertDialogDescription = (String) hvDocConfig.get(FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_DESCRIPTION);
        if (shouldShowCloseAlert != null) {
            getDocConfig().setShouldShowCloseAlert(shouldShowCloseAlert);
            if (shouldShowCloseAlert) {
                if (closeAlertDialogTitle != null && !closeAlertDialogTitle.isEmpty()) {
                    getDocConfig().setCloseAlertDialogTitle(closeAlertDialogTitle);
                }
                if (closeAlertDialogDescription != null && !closeAlertDialogDescription.isEmpty()) {
                    getDocConfig().setCloseAlertDialogDesc(closeAlertDialogDescription);
                }
            }
        }

        Boolean shouldDocAutoCapture = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_AUTO_CAPTURE);
        Integer autoCaptureDuration = (Integer) hvDocConfig.get(FLHyperSnapConstants.DOC_AUTO_CAPTURE_DURATION);
        if (shouldDocAutoCapture != null) {
//            getDocConfig().setShouldAutoCapture(shouldDocAutoCapture);
            if (shouldDocAutoCapture && autoCaptureDuration != null) {
//                getDocConfig().setAutoCaptureDuration(autoCaptureDuration);
            }
        }

        Boolean shouldReadBarcode = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_READ_BARCODE);
        Integer readBarcodeTimeout = (Integer) hvDocConfig.get(FLHyperSnapConstants.DOC_READ_BARCODE_TIMEOUT);
        if (shouldReadBarcode != null) {
            getDocConfig().setShouldReadBarcode(shouldReadBarcode);
            if (shouldReadBarcode && readBarcodeTimeout != null) {
                getDocConfig().setReadBarcodeTimeout(readBarcodeTimeout);
            }
        }

        Map<String, Object> ocrApiDetails = (Map<String, Object>) hvDocConfig.get(FLHyperSnapConstants.OCR_API_DETAILS);
        if (ocrApiDetails != null && !ocrApiDetails.isEmpty()) {
            setOCRAPIDetails(ocrApiDetails);
        }

        Boolean shouldShowReviewScreen = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_REVIEW_SCREEN);
        if (shouldShowReviewScreen != null) {
            getDocConfig().setShouldShowReviewScreen(shouldShowReviewScreen);
        }

        Boolean shouldEnableRetries = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_ENABLE_RETRIES);
        if (shouldEnableRetries != null) {
            getDocConfig().setShouldEnableRetries(shouldEnableRetries);
        }
    }

    private void setUIStrings(Map<String, Object> uiStringsMap) {
        //remove null ui strings
        uiStringsMap.values().removeAll(Collections.singleton(null));

        try {
            JSONObject uiStringsJsonObject = FLHVUtils.convertMapToJson(uiStringsMap);
            // Using customUIStrings
            getDocConfig().setCustomUIStrings(uiStringsJsonObject);
        } catch (JSONException e) {
            Log.e(TAG, e.getMessage());
        }
    }

    @SuppressWarnings("unchecked")
    private void setOCRAPIDetails(Map<String, Object> ocrApiDetails) {
        String ocrEndpoint = (String) ocrApiDetails.get(FLHyperSnapConstants.OCR_ENDPOINT);
        String documentSideString = (String) ocrApiDetails.get(FLHyperSnapConstants.DOCUMENT_SIDE_STRING);
        Map<String, Object> params = (Map<String, Object>) ocrApiDetails.get(FLHyperSnapConstants.PARAMS);
        Map<String, Object> headers = (Map<String, Object>) ocrApiDetails.get(FLHyperSnapConstants.HEADERS);

        HVDocConfig.DocumentSide documentSide = HVDocConfig.DocumentSide.valueOf(documentSideString);
        JSONObject paramsObj = new JSONObject();
        JSONObject headersObj = new JSONObject();

        try {
            if (params != null) {
                paramsObj = FLHVUtils.convertMapToJson(params);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        try {
            if (headers != null) {
                headersObj = FLHVUtils.convertMapToJson(headers);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }

        getDocConfig().setOCRDetails(ocrEndpoint,
                documentSide,
                paramsObj.toString(),
                headersObj.toString()
        );
    }

    public void start(Context context, final DocCaptureCompletionHandler callback) {
        HVDocsActivity.start(context, getDocConfig(), callback);
    }
}
