package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Collections;
import java.util.Map;
import java.util.Objects;

import co.hyperverge.hypersnapsdk.activities.HVFaceActivity;
import co.hyperverge.hypersnapsdk.listeners.FaceCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVFaceConfig;


public class FLHVFaceCapture {

    private static final String TAG = FLHVFaceCapture.class.getSimpleName();
    private HVFaceConfig faceConfig;

    public void createNewFaceConfig() {
        this.faceConfig = new HVFaceConfig();
    }

    public HVFaceConfig getFaceConfig() {
        if (this.faceConfig == null) {
            this.faceConfig = new HVFaceConfig();
        }
        return this.faceConfig;
    }

    @SuppressWarnings("unchecked")
    public void setHVFaceConfig(Map<String, Object> hvFaceConfig) {
        Map<String, Object> faceCaptureUIStrings = (Map<String, Object>) hvFaceConfig.get(FLHyperSnapConstants.FACE_CAPTURE_UI_STRINGS);
        if (faceCaptureUIStrings != null && !faceCaptureUIStrings.isEmpty()) {
            setUIStrings(faceCaptureUIStrings);
        }

        Boolean shouldShowTrustLogos = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_SHOW_TRUST_LOGOS);
        if (shouldShowTrustLogos != null) {
            getFaceConfig().setShowTrustLogos(shouldShowTrustLogos);
        }

        Boolean shouldShowInstructionPage = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_SHOW_INSTRUCTION_PAGE);
        if (shouldShowInstructionPage != null) {
            getFaceConfig().setShouldShowInstructionPage(shouldShowInstructionPage);
        }

        Boolean shouldShowCameraSwitchButton = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_SHOW_CAMERA_SWITCH_BUTTON);
        if (shouldShowCameraSwitchButton != null) {
            getFaceConfig().setShouldShowCameraSwitchButton(shouldShowCameraSwitchButton);
        }

        Boolean shouldUseBackCamera = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_USE_BACK_CAMERA);
        if (shouldUseBackCamera != null) {
            getFaceConfig().setShouldUseBackCamera(shouldUseBackCamera);
        }

        Boolean shouldAddPadding = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_ADD_PADDING);
        if (shouldAddPadding != null) {
            getFaceConfig().setShouldEnablePadding(shouldAddPadding);
        }

        Map<String, Object> facePadding = (Map<String, Object>) hvFaceConfig.get(FLHyperSnapConstants.FACE_PADDING);
        if (facePadding != null && !facePadding.isEmpty()) {
            Double paddingLeft = (Double) facePadding.get(FLHyperSnapConstants.PADDING_LEFT);
            Double paddingRight = (Double) facePadding.get(FLHyperSnapConstants.PADDING_RIGHT);
            Double paddingTop = (Double) facePadding.get(FLHyperSnapConstants.PADDING_TOP);
            Double paddingBottom = (Double) facePadding.get(FLHyperSnapConstants.PADDING_BOTTOM);

            setPadding(paddingLeft, paddingRight, paddingTop, paddingBottom);
        }

        Boolean shouldShowCloseAlert = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_SHOW_CLOSE_ALERT);
        String closeAlertDialogTitle = (String) hvFaceConfig.get(FLHyperSnapConstants.FACE_CLOSE_ALERT_DIALOG_TITLE);
        String closeAlertDialogDesc = (String) hvFaceConfig.get(FLHyperSnapConstants.FACE_CLOSE_ALERT_DIALOG_DESC);
        if (shouldShowCloseAlert != null) {
            getFaceConfig().setShouldShowCloseAlert(shouldShowCloseAlert);
            if (shouldShowCloseAlert) {
                if (closeAlertDialogTitle != null && !closeAlertDialogTitle.isEmpty()) {
                    getFaceConfig().setCloseAlertDialogTitle(closeAlertDialogTitle);
                }
                if (closeAlertDialogDesc != null && !closeAlertDialogDesc.isEmpty()) {
                    getFaceConfig().setCloseAlertDialogDesc(closeAlertDialogDesc);
                }
            }
        }

        Boolean shouldFaceAutoCapture = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_AUTO_CAPTURE);
        Integer autoCaptureDuration = (Integer) hvFaceConfig.get(FLHyperSnapConstants.AUTO_CAPTURE_DURATION);
        if (shouldFaceAutoCapture != null) {
            getFaceConfig().setShouldAutoCapture(shouldFaceAutoCapture);
            if (shouldFaceAutoCapture && autoCaptureDuration != null) {
                getFaceConfig().setAutoCaptureDuration(autoCaptureDuration);
            }
        }

        Map<String, Object> faceVideoRecording = (Map<String, Object>) hvFaceConfig.get(FLHyperSnapConstants.FACE_VIDEO_RECORDING);
        if (faceVideoRecording != null && !faceVideoRecording.isEmpty()) {
            Boolean shouldRecordVideo = (Boolean) faceVideoRecording.get(FLHyperSnapConstants.SHOULD_RECORD_VIDEO);
            Integer noOfFrames = (Integer) faceVideoRecording.get(FLHyperSnapConstants.NO_OF_FRAMES);
            Double bitRateM = (Double) faceVideoRecording.get(FLHyperSnapConstants.BIT_RATE_M);
            Integer fps = (Integer) faceVideoRecording.get(FLHyperSnapConstants.FPS);
            setVideoRecording(Boolean.TRUE.equals(shouldRecordVideo), noOfFrames, bitRateM, fps);
        }

        Boolean shouldCheckActiveLiveness = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.CHECK_ACTIVE_LIVENESS);
        Integer totalGestures = (Integer) hvFaceConfig.get(FLHyperSnapConstants.TOTAL_GESTURES);
        if (shouldCheckActiveLiveness != null) {
            getFaceConfig().setShouldCheckActiveLiveness(shouldCheckActiveLiveness);
            if (shouldCheckActiveLiveness && totalGestures != null) {
                getFaceConfig().setTotalGestures(totalGestures);
            }
        }

        Map<String, Object> livenessApiDetails = (Map<String, Object>) hvFaceConfig.get(FLHyperSnapConstants.LIVENESS_API_DETAILS);
        if (livenessApiDetails != null && !livenessApiDetails.isEmpty()) {
            String livenessEndpoint = (String) livenessApiDetails.get(FLHyperSnapConstants.LIVENESS_ENDPOINT);
            String livenessValue = (String) livenessApiDetails.get(FLHyperSnapConstants.LIVENESS_VALUE);
            Map<String, Object> livenessParams = (Map<String, Object>) livenessApiDetails.get(FLHyperSnapConstants.LIVENESS_PARAMS);
            Map<String, Object> livenessHeaders = (Map<String, Object>) livenessApiDetails.get(FLHyperSnapConstants.LIVENESS_HEADERS);

            setLivenessEndpoint(livenessEndpoint);
            setLivenessMode(livenessValue);
            setLivenessAPIParameters(livenessParams);
            setLivenessAPIHeaders(livenessHeaders);
        }

        Boolean shouldReturnFullImageUrl = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_RETURN_FULL_IMAGE_URL);
        if (shouldReturnFullImageUrl != null) {
            getFaceConfig().setShouldReturnFullImageUrl(shouldReturnFullImageUrl);
        }

        Boolean shouldHandleRetries = (Boolean) hvFaceConfig.get(FLHyperSnapConstants.FACE_SHOULD_HANDLE_RETRIES);
        if (shouldHandleRetries != null) {
            getFaceConfig().setShouldHandleRetries(shouldHandleRetries);
        }
    }

    public void setPadding(Double leftPadding, Double rightPadding, Double topPadding, Double bottomPadding) {
        getFaceConfig().setPadding(leftPadding.floatValue(), rightPadding.floatValue(), topPadding.floatValue(), bottomPadding.floatValue());
    }

    private void setUIStrings(Map<String, Object> uiStringsMap) {
        //remove null ui strings
        uiStringsMap.values().removeAll(Collections.singleton(null));

        try {
            JSONObject uiStringsJsonObject = FLHVUtils.convertMapToJson(uiStringsMap);
            // Using customUIStrings
            getFaceConfig().setCustomUIStrings(uiStringsJsonObject);
        } catch (JSONException e) {
            Log.e(TAG, e.getMessage());
        }
    }

    public void setVideoRecording(boolean shouldRecordVideo, Integer numberOfFrames, Double bitrateM, Integer fps) {
        if (shouldRecordVideo) {
            if (bitrateM != null) getFaceConfig().setBitrateM(bitrateM);
            if (numberOfFrames != null) getFaceConfig().setNumberOfFrames(numberOfFrames);
            if (fps != null) getFaceConfig().setFps(fps);
        }
        getFaceConfig().setShouldRecordVideo(shouldRecordVideo);
    }

    public void setLivenessEndpoint(String livenessEndpoint) {
        getFaceConfig().setLivenessEndpoint(livenessEndpoint);
    }

    public void setLivenessMode(String livenessValue) {
        try {
            HVFaceConfig.LivenessMode livenessMode = HVFaceConfig.LivenessMode.valueOf(livenessValue);
            getFaceConfig().setLivenessMode(livenessMode);
        } catch (Exception exp) {
            Log.e(TAG, Objects.requireNonNull(exp.getMessage()));
        }
    }

    public void setLivenessAPIParameters(Map<String, Object> params) {
        JSONObject paramsObj = new JSONObject();
        try {
            if (params != null) {
                paramsObj = FLHVUtils.convertMapToJson(params);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        getFaceConfig().setLivenessAPIParameters(paramsObj);
    }

    public void setLivenessAPIHeaders(Map<String, Object> headers) {
        JSONObject headersObj = new JSONObject();
        try {
            if (headers != null) {
                headersObj = FLHVUtils.convertMapToJson(headers);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        getFaceConfig().setLivenessAPIHeaders(headersObj);
    }

    public void start(Context context, final FaceCaptureCompletionHandler callback) {
        HVFaceActivity.start(context, getFaceConfig(), callback);
    }
}
