package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.util.Log;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import co.hyperverge.hypersnapsdk.objects.HVBaseResponse;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVResponse;

public class FLHVUtils {
    public static final String TAG = "FLHVUtils";

    public static JSONObject convertMapToJson(Map<String, Object> map) throws JSONException {
        JSONObject object = new JSONObject();
        ObjectMapper objectMapper = new ObjectMapper();

        try {
            String json = objectMapper.writeValueAsString(map);
            object = new JSONObject(json);
        } catch (JsonProcessingException e) {
            Log.e(TAG, e.getMessage());
        }
        return object;
    }

    public static Map<String, Object> convertJsonToMap(JSONObject jsonObject) {
        ObjectMapper mapper = new ObjectMapper();
        String jsonString = jsonObject.toString();
        Map<String, Object> map = new HashMap<>();
        try {
            map = mapper.readValue(jsonString, new TypeReference<Map<String, Object>>() {
            });
        } catch (IOException e) {
            Log.e(TAG, e.getMessage());
        }
        return map;
    }

    public static ArrayList<Map<String, Object>> getFailedResponsesList(List<HVBaseResponse> retakeAttemptResponses) {
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();
        try {
            if (retakeAttemptResponses != null) {
                for (HVBaseResponse response : retakeAttemptResponses) {
                    HashMap<String, Object> responseMap = new HashMap<>();
                    arrayList.add(convertHVResponseToMap(response, responseMap));
                }
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        return arrayList;
    }

    public static List<Map<String, Object>> getMapsFromOnResult(HVError error, HVResponse hvResponse) {
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();
        Map<String, Object> errorObj = new HashMap<>();
        Map<String, Object> resultObj = new HashMap<>();

        try {
            if (error != null) {
                errorObj.put("errorCode", error.getErrorCode());
                errorObj.put("errorMessage", error.getErrorMessage());
            }
            if (hvResponse != null) {

                JSONObject result = hvResponse.getApiResult();
                JSONObject headers = hvResponse.getApiHeaders();

                String imageURI = hvResponse.getImageURI();
                String fullImageUri = hvResponse.getFullImageURI();
                String action = hvResponse.getAction();
                String retakeMessage = hvResponse.getRetakeMessage();
                String videoUri = hvResponse.getVideoUri();
                Map<String, List<String>> gestureLivenessUrls = hvResponse.getGestureLivenessImageUrls();
                String rawBarcode = hvResponse.getRawBarcode();
                ArrayList<Map<String, Object>> retakeAttemptResponses = FLHVUtils.getFailedResponsesList(hvResponse.getRetakeAttemptResponses());

                if (imageURI != null && !imageURI.isEmpty()) {
                    resultObj.put("imageUri", imageURI);
                }
                if (videoUri != null && !videoUri.isEmpty()) {
                    resultObj.put("videoUri", videoUri);
                }
                if (fullImageUri != null && !fullImageUri.isEmpty()) {
                    resultObj.put("fullImageUri", fullImageUri);
                }
                if (retakeMessage != null && !retakeMessage.isEmpty()) {
                    resultObj.put("retakeMessage", retakeMessage);
                }
                if (action != null && !action.isEmpty()) {
                    resultObj.put("action", action);
                }
                if (gestureLivenessUrls != null && !gestureLivenessUrls.isEmpty()) {
                    resultObj.put("gestureLivenessUrls", gestureLivenessUrls);
                }
                if (rawBarcode != null) {
                    resultObj.put("rawBarcode", rawBarcode);
                }
                if (!retakeAttemptResponses.isEmpty()) {
                    resultObj.put("retakeAttemptResponses", retakeAttemptResponses);
                }
                if (result != null) {
                    resultObj.put("apiResult", result.toString());
                }
                if (headers != null) {
                    resultObj.put("apiHeaders", headers.toString());
                }
            }

        } catch (Exception e) {
            Log.e(TAG, (e.getMessage()));

        }
        arrayList.add(errorObj);
        arrayList.add(resultObj);
        return arrayList;
    }

    public static List<Map<String, Object>> getArrayListFromOnResultNetworkHelper(HVError error, HVResponse hvResponse) {
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();
        Map<String, Object> errorObj = new HashMap<>();
        Map<String, Object> resultsObj = new HashMap<>();

        try {
            if (error != null) {
                errorObj.put("errorCode", error.getErrorCode());
                errorObj.put("errorMessage", error.getErrorMessage());
            }
            if (hvResponse != null) {
                JSONObject result = hvResponse.getApiResult();
                JSONObject headers = hvResponse.getApiHeaders();

                String imageURI = hvResponse.getImageURI();
                String fullImageUri = hvResponse.getFullImageURI();
                String action = hvResponse.getAction();
                String retakeMessage = hvResponse.getRetakeMessage();

                if (imageURI != null && !imageURI.isEmpty()) {
                    resultsObj.put("imageUri", imageURI);
                }
                if (fullImageUri != null && !fullImageUri.isEmpty()) {
                    resultsObj.put("fullImageUri", fullImageUri);
                }
                if (retakeMessage != null && !retakeMessage.isEmpty()) {
                    resultsObj.put("retakeMessage", retakeMessage);
                }
                if (action != null && !action.isEmpty()) {
                    resultsObj.put("action", action);
                }
                if (result != null) {
                    resultsObj.put("apiResult", result.toString());
                }
                if (headers != null) {
                    resultsObj.put("apiHeaders", headers.toString());
                }
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        arrayList.add(errorObj);
        arrayList.add(resultsObj);
        return arrayList;
    }

    public static List<Map<String, Object>> getALFromOnResultQRScan(HVError error, JSONObject result) {
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();

        Map<String, Object> errorObj = new HashMap<>();
        Map<String, Object> resultsObj = new HashMap<>();
        try {
            if (error != null) {
                errorObj.put("errorCode", error.getErrorCode());
                errorObj.put("errorMessage", error.getErrorMessage());
            }
            if (result != null) {
                resultsObj = FLHVUtils.convertJsonToMap(result);
            }

        } catch (Exception e) {
            Log.e(TAG, Objects.requireNonNull(e.getMessage()));
        }
        arrayList.add(errorObj);
        arrayList.add(resultsObj);
        return arrayList;
    }

    public static HashMap<String, Object> convertHVResponseToMap(HVBaseResponse hvResponse, HashMap<String, Object> resultsObj) {

        JSONObject apiResult = hvResponse.getApiResult();
        JSONObject apiHeaders = hvResponse.getApiHeaders();

        String imageURI = hvResponse.getImageURI();
        String fullImageUri = hvResponse.getFullImageURI();
        String action = hvResponse.getAction();
        String retakeMessage = hvResponse.getRetakeMessage();
        String videoUri = hvResponse.getVideoUri();
        Map<String, List<String>> gestureLivenessUrls = hvResponse.getGestureLivenessImageUrls();
        String rawBarcode = hvResponse.getRawBarcode();

        try {
            if (imageURI != null && !imageURI.isEmpty()) {
                resultsObj.put("imageUri", imageURI);
            }
            if (fullImageUri != null && !fullImageUri.isEmpty()) {
                resultsObj.put("fullImageUri", fullImageUri);
            }
            if (retakeMessage != null && !retakeMessage.isEmpty()) {
                resultsObj.put("retakeMessage", retakeMessage);
            }
            if (action != null && !action.isEmpty()) {
                resultsObj.put("action", action);
            }
            if (videoUri != null && !videoUri.isEmpty()) {
                resultsObj.put("videoUri", videoUri);
            }
            if (gestureLivenessUrls != null && !gestureLivenessUrls.isEmpty()) {
                resultsObj.put("gestureLivenessUrls", gestureLivenessUrls);
            }
            if (rawBarcode != null) {
                resultsObj.put("rawBarcode", rawBarcode);
            }
            if (apiResult != null) {
                resultsObj.put("apiResult", apiResult.toString());
            }
            if (apiHeaders != null) {
                resultsObj.put("apiHeaders", apiHeaders.toString());
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        return resultsObj;
    }
}
