package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import co.hyperverge.hypersnapsdk.listeners.APICompletionCallback;
import co.hyperverge.hypersnapsdk.listeners.DocCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.listeners.FaceCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.listeners.QRScannerCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVResponse;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;


/**
 * HypersnapsdkFlutterPlugin
 */
public class HypersnapsdkFlutterPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    /// classes
    private final FLHyperSnapSDK flHyperSnapSDK = new FLHyperSnapSDK();
    private final FLHVDocsCapture flhvDocsCapture = new FLHVDocsCapture();
    private final FLHVFaceCapture flhvFaceCapture = new FLHVFaceCapture();
    private final FLHVNetworkHelper flhvNetworkHelper = new FLHVNetworkHelper();
    private final FLHVQRScanCapture flhvqrScanCapture = new FLHVQRScanCapture();

    //channel specifics
    private MethodChannel hyperSnapSDKChannel;
    protected Context context;
    protected Activity activity;

    private static final String ERROR_OBJ = "errorObj";
    private static final String RESULT_OBJ = "resultObj";

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        this.context = flutterPluginBinding.getApplicationContext();
        hyperSnapSDKChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), FLHyperSnapConstants.METHOD_CHANNEL_STRING);
        hyperSnapSDKChannel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        switch (call.method) {
            case FLHyperSnapConstants.INITIALIZE_METHOD: {
                String appId = call.argument(FLHyperSnapConstants.APP_ID_ARG);
                String appKey = call.argument(FLHyperSnapConstants.APP_KEY_ARG);
                String region = call.argument(FLHyperSnapConstants.REGION_ARG);
                flHyperSnapSDK.initialize(context, appId, appKey, region);
                result.success("Initialized");
                break;
            }
            case FLHyperSnapConstants.INITIALIZE_WITH_TOKEN_METHOD: {
                String appId = call.argument(FLHyperSnapConstants.APP_ID_ARG);
                String accessToken = call.argument(FLHyperSnapConstants.ACCESS_TOKEN_ARG);
                String region = call.argument(FLHyperSnapConstants.REGION_ARG);
                flHyperSnapSDK.initializeWithToken(context, appId, accessToken, region);
                result.success("Initialized");
                break;
            }
            case FLHyperSnapConstants.HYPER_SNAP_SDK_CONFIG_METHOD:
                Boolean shouldEnableSSLPinning = call.argument(FLHyperSnapConstants.SHOULD_ENABLE_SSL_PINNING_ARG);
                Boolean shouldUseSignature = call.argument(FLHyperSnapConstants.SHOULD_USE_SIGNATURE_ARG);
                Boolean shouldReturnRawResponse = call.argument(FLHyperSnapConstants.SHOULD_RETURN_RAW_RESPONSE_ARG);
                Integer connectTimeout = call.argument(FLHyperSnapConstants.CONNECT_TIMEOUT_ARG);
                Integer readTimeout = call.argument(FLHyperSnapConstants.READ_TIMEOUT_ARG);
                Integer writeTimeout = call.argument(FLHyperSnapConstants.WRITE_TIMEOUT_ARG);
                String uiConfigMap = call.argument(FLHyperSnapConstants.UI_CONFIG_MAP);

                flHyperSnapSDK.setHyperSnapConfig(context, shouldEnableSSLPinning, shouldUseSignature, shouldReturnRawResponse, connectTimeout, readTimeout, writeTimeout, uiConfigMap);
                result.success("sdk config set");
                break;
            case FLHyperSnapConstants.START_USER_SESSION_METHOD:
                String userId = call.argument(FLHyperSnapConstants.USER_ID_ARG);
                boolean isUserSessionStarted = flHyperSnapSDK.startUserSession(userId);
                result.success(isUserSessionStarted);
                break;
            case FLHyperSnapConstants.END_USER_SESSION_METHOD:
                flHyperSnapSDK.endUserSession();
                result.success("user session ended");
                break;
            case FLHyperSnapConstants.SORT_JSON_KEYS_ALPHABETICALLY_METHOD:
                Map<String, Object> map = call.argument("map");
                result.success(flHyperSnapSDK.sortJSONKeysAlphabetically(map));
                break;
            case FLHyperSnapConstants.START_DOC_CAPTURE:
                Map<String, Object> docConfig = call.argument(FLHyperSnapConstants.HV_DOC_CONFIG);
                flhvDocsCapture.createNewDocConfig();
                if (docConfig != null) flhvDocsCapture.setHVDocConfig(docConfig);
                DocCaptureCompletionHandler docCaptureCallback = new DocCaptureCompletionHandler() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) FLHVUtils.getMapsFromOnResult(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                flhvDocsCapture.start(activity, docCaptureCallback);
                break;
            case FLHyperSnapConstants.START_FACE_CAPTURE:
                Map<String, Object> faceConfig = call.argument(FLHyperSnapConstants.HV_FACE_CONFIG);
                flhvFaceCapture.createNewFaceConfig();
                if (faceConfig != null) flhvFaceCapture.setHVFaceConfig(faceConfig);
                FaceCaptureCompletionHandler faceCaptureCallback = new FaceCaptureCompletionHandler() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) FLHVUtils.getMapsFromOnResult(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                flhvFaceCapture.start(activity, faceCaptureCallback);
                break;
            case FLHyperSnapConstants.NETWORK_HELPER_MAKE_OCR_CALL:
                APICompletionCallback ocrCallback = new APICompletionCallback() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) FLHVUtils.getArrayListFromOnResultNetworkHelper(hvError, hvResponse);

                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                String endpoint = call.argument(FLHyperSnapConstants.NETWORK_HELPER_OCR_ENDPOINT);
                String documentUri = call.argument(FLHyperSnapConstants.NETWORK_HELPER_OCR_DOCUMENT_URI);
                Map<String, Object> parameters = call.argument(FLHyperSnapConstants.NETWORK_HELPER_OCR_PARAMETERS);
                Map<String, Object> headers = call.argument(FLHyperSnapConstants.NETWORK_HELPER_OCR_HEADERS);
                flhvNetworkHelper.makeOCRCall(activity, endpoint, documentUri, parameters, headers, ocrCallback);
                break;
            case FLHyperSnapConstants.NETWORK_HELPER_MAKE_FACE_MATCH_CALL:
                APICompletionCallback faceMatchCallback = new APICompletionCallback() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) FLHVUtils.getArrayListFromOnResultNetworkHelper(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                String endpoint1 = call.argument(FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_ENDPOINT);
                String documentUri1 = call.argument(FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_DOCUMENT_URI);
                String faceUri = call.argument(FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_FACE_URI);
                String faceMatchMode = call.argument(FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_FACE_MATCH_MODE);
                Map<String, Object> parameters1 = call.argument(FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_PARAMETERS);
                Map<String, Object> headers1 = call.argument(FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_HEADERS);
                flhvNetworkHelper.makeFaceMatchCall(activity, endpoint1, documentUri1, faceUri, faceMatchMode, parameters1, headers1, faceMatchCallback);
                break;
            case FLHyperSnapConstants.QR_SCAN_CAPTURE_START:
                Map<String, Object> qrConfig = call.argument(FLHyperSnapConstants.HV_QR_CONFIG);
                flhvqrScanCapture.createNewConfig();
                if (qrConfig != null) flhvqrScanCapture.setHVQRConfig(qrConfig);
                QRScannerCompletionHandler qrScanCallback = new QRScannerCompletionHandler() {
                    @Override
                    public void onResult(HVError hvError, JSONObject hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) FLHVUtils.getALFromOnResultQRScan(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                flhvqrScanCapture.start(activity, qrScanCallback);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        hyperSnapSDKChannel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }
}
