import Foundation
import HyperSnapSDK

public class FLHVDocsCapture {

    var docConfig = HyperSnapSDK.HVDocConfig()

    public func getDocConfig() -> HVDocConfig {
        return self.docConfig
    }

    public func createNewDocConfig() {
        docConfig = HyperSnapSDK.HVDocConfig()
    }

    public func setHVDocConfig(hvDocConfig: [String: Any]) {
        if let uiStrings = (hvDocConfig[FLHyperSnapConstants.DOC_CAPTURE_UI_STRINGS] as? [String: Any]) {
            if !uiStrings.isEmpty {
                setUIStrings(uiStrings: uiStrings)
            }
        }

        if let shouldShowTrustLogos = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_TRUST_LOGOS] as? Bool) {
            getDocConfig().setShowTrustLogos(shouldShowTrustLogos)
        }

        if let shouldShowInstructionPage = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_INSTRUCTION_PAGE] as? Bool) {
            getDocConfig().setShouldShowInstructionsPage(shouldShowInstructionPage)
        }

        if let shouldEnableDocUpload = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_ENABLE_UPLOAD] as? Bool) {
            getDocConfig().setEnableDocumentUpload(shouldEnableDocUpload)
        }

        if let shouldShowFlashIcon = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_FLASH_ICON] as? Bool) {
            getDocConfig().setShouldShowFlashButton(shouldShowFlashIcon)
        }

        if let docType = (hvDocConfig[FLHyperSnapConstants.DOC_TYPE_VALUE] as? String) {
            let documentType = getDocumentType(document: docType)
            getDocConfig().setDocumentType(documentType)
        }

        if let aspectRatioValue = (hvDocConfig[FLHyperSnapConstants.DOC_ASPECT_RATIO_VALUE] as? Double) {
            getDocConfig().setAspectRatio(aspectRatioValue)
        }

        if let shouldSetPadding = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SET_PADDING] as? Bool) {
            getDocConfig().setShouldAddPadding(shouldSetPadding)
        }

        if let defaultPaddingValue = (hvDocConfig[FLHyperSnapConstants.DOC_DEFAULT_PADDING_VALUE] as? Double) {
            getDocConfig().setDefaultPadding(defaultPaddingValue)
        }

        if let shouldShowCloseAlert = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_CLOSE_ALERT] as? Bool) {
            getDocConfig().setShouldShowCloseAlert(shouldShowCloseAlert)
            if shouldShowCloseAlert {
                if let closeAlertDialogTitle = (hvDocConfig[FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_TITLE] as? String) {
                    if !closeAlertDialogTitle.isEmpty {
                        getDocConfig().textConfig.setCloseAlertTitle(closeAlertDialogTitle)
                    }
                }
                if let closeAlertDialogDescription = (hvDocConfig[FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_DESCRIPTION] as? String) {
                    if !closeAlertDialogDescription.isEmpty {
                        getDocConfig().textConfig.setCloseAlertDesc(closeAlertDialogDescription)
                    }
                }
            }
        }

        if let shouldAutoCapture = (hvDocConfig[FLHyperSnapConstants.DOC_AUTO_CAPTURE] as? Bool) {
            //  getDocConfig().setShouldAutoCapture(shouldAutoCapture)
            if shouldAutoCapture {
                if let duration = (hvDocConfig[FLHyperSnapConstants.DOC_AUTO_CAPTURE_DURATION] as? Int) {
                    //  getDocConfig().setAutoCaptureDuration(durationInMS: duration)
                }
            }
        }
        
        if let shouldReadBarcode = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_READ_BARCODE] as? Bool) {
            getDocConfig().setShouldReadBarcode(shouldReadBarcode)
            if shouldReadBarcode {
                if let readBarcodeTimeout = (hvDocConfig[FLHyperSnapConstants.DOC_READ_BARCODE_TIMEOUT] as? Int) {
                    getDocConfig().setReadBarcodeTimeout(durationInMS: readBarcodeTimeout)
                }
            }
        }

        if let ocrApiDetails = (hvDocConfig[FLHyperSnapConstants.OCR_API_DETAILS] as? [String: Any]) {
            if !ocrApiDetails.isEmpty {
                setOCRAPIDetails(ocrApiDetails: ocrApiDetails)
            }
        }

        if let shouldShowReviewScreen = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_SHOW_REVIEW_SCREEN] as? Bool) {
            getDocConfig().setShouldShowReviewPage(shouldShowReviewScreen)
        }

        if let shouldHandleRetries = (hvDocConfig[FLHyperSnapConstants.DOC_SHOULD_ENABLE_RETRIES] as? Bool) {
            getDocConfig().setShouldHandleRetries(shouldHandleRetries)
        }
    }

    public func setOCRAPIDetails(ocrApiDetails: [String: Any]) {
        let ocrEndpoint = ocrApiDetails[FLHyperSnapConstants.OCR_ENDPOINT] as! String
        let documentSideString = ocrApiDetails[FLHyperSnapConstants.DOCUMENT_SIDE_STRING] as! String
        let documentSide = getDocumentSide(documentSide: documentSideString)
        let params = ocrApiDetails[FLHyperSnapConstants.PARAMS] as? [String: Any] ?? [String: Any]()
        let headers = ocrApiDetails[FLHyperSnapConstants.HEADERS] as? [String: Any] ?? [String: Any]()

        getDocConfig().setOCRAPIDetails(
            ocrEndpoint,
            documentSide: documentSide,
            params: params as [String: AnyObject],
            headers: headers as? [String: String]
        )

    }

    public func setUIStrings(uiStrings: [String: Any]) {
        var uiStringsUpdated: [String: String] = [:]
        for (key, value) in uiStrings {
            if let stringValue = value as? String {
                uiStringsUpdated[key] = stringValue
            }
        }
        getDocConfig().textConfig.setTextConfig(uiStringsUpdated)
    }

    public func start(
        context: UIViewController,
        callback: @escaping ((HVError?, HVResponse?, UIViewController) -> Void)
    ) {
        HVDocsViewController.start(
            context,
            hvDocConfig: getDocConfig(),
            completionHandler: callback
        )
    }

    public func getDocumentSide(documentSide: String) -> HVDocConfig.DocumentSide {
        switch documentSide.lowercased() {
        case "front":
            return HVDocConfig.DocumentSide.front
        case "back":
            return HVDocConfig.DocumentSide.back
        default:
            return HVDocConfig.DocumentSide.front
        }
    }

    public func getDocumentType(document: String) -> HyperSnapParams.DocumentType {
        switch document.lowercased() {
        case "a4":
            return HyperSnapParams.DocumentType.a4
        case "card":
            return HyperSnapParams.DocumentType.card
        case "other":
            return HyperSnapParams.DocumentType.other
        case "passport":
            return HyperSnapParams.DocumentType.passport
        default:
            return HyperSnapParams.DocumentType.card
        }
    }
}
