import Foundation
import HyperSnapSDK

public class FLHVFaceCapture {

    var faceConfig = HVFaceConfig()

    public func getFaceConfig() -> HVFaceConfig {
        return faceConfig
    }

    public func createNewFaceConfig() {
        faceConfig = HyperSnapSDK.HVFaceConfig()
    }

    public func setHVFaceConfig(hvFaceConfig: [String: Any]) {
        if let uiStrings = (hvFaceConfig[FLHyperSnapConstants.FACE_CAPTURE_UI_STRINGS] as? [String: Any]) {
            if !uiStrings.isEmpty {
                setUIStrings(uiStrings: uiStrings)
            }
        }

        if let shouldShowTrustLogos = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_TRUST_LOGOS] as? Bool) {
            getFaceConfig().setShowTrustLogos(shouldShowTrustLogos)
        }

        if let shouldShowInstructionPage = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_INSTRUCTION_PAGE] as? Bool) {
            getFaceConfig().setShouldShowInstructionsPage(shouldShowInstructionPage)
        }

        if let shouldShowCameraSwitchButton = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_CAMERA_SWITCH_BUTTON] as? Bool) {
            getFaceConfig().setShouldShowCameraSwitchButton(shouldShowCameraSwitchButton)
        }

        if let shouldUseBackCamera = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_USE_BACK_CAMERA] as? Bool) {
            getFaceConfig().setShouldUseBackCamera(shouldUseBackCamera)
        }

        if let shouldAddPadding = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_ADD_PADDING] as? Bool) {
            getFaceConfig().setShouldEnablePadding(shouldAddPadding)
        }

        if let facePadding = (hvFaceConfig[FLHyperSnapConstants.FACE_PADDING] as? [String: Any]) {
            if !facePadding.isEmpty {
                let paddingLeft = facePadding[FLHyperSnapConstants.PADDING_LEFT] as! Double
                let paddingRight = facePadding[FLHyperSnapConstants.PADDING_RIGHT] as! Double
                let paddingTop = facePadding[FLHyperSnapConstants.PADDING_TOP] as! Double
                let paddingBottom = facePadding[FLHyperSnapConstants.PADDING_BOTTOM] as! Double
                setPadding(leftPadding: paddingLeft, rightPadding: paddingRight, topPadding: paddingTop, bottomPadding: paddingBottom)
            }
        }

        if let shouldShowCloseAlert = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_SHOW_CLOSE_ALERT] as? Bool) {
            getFaceConfig().setShouldShowCloseAlert(shouldShowCloseAlert)
            if shouldShowCloseAlert {
                if let closeAlertDialogTitle = (hvFaceConfig[FLHyperSnapConstants.FACE_CLOSE_ALERT_DIALOG_TITLE] as? String) {
                    if !closeAlertDialogTitle.isEmpty {
                        getFaceConfig().textConfig.setCloseAlertTitle(closeAlertDialogTitle)
                    }
                }
                if let closeAlertDialogDescription = (hvFaceConfig[FLHyperSnapConstants.FACE_CLOSE_ALERT_DIALOG_DESC] as? String) {
                    if !closeAlertDialogDescription.isEmpty {
                        getFaceConfig().textConfig.setCloseAlertDesc(closeAlertDialogDescription)
                    }
                }
            }
        }

        if let shouldAutoCapture = (hvFaceConfig[FLHyperSnapConstants.FACE_AUTO_CAPTURE] as? Bool) {
            getFaceConfig().setShouldAutoCapture(shouldAutoCapture)
            if shouldAutoCapture {
                if let duration = (hvFaceConfig[FLHyperSnapConstants.AUTO_CAPTURE_DURATION] as? Int) {
                    getFaceConfig().setAutoCaptureDuration(duration)
                }
            }
        }

        if let faceVideoRecording = (hvFaceConfig[FLHyperSnapConstants.FACE_VIDEO_RECORDING] as? [String: Any]) {
            if !faceVideoRecording.isEmpty {
                if let shouldRecordVideo = faceVideoRecording[FLHyperSnapConstants.SHOULD_RECORD_VIDEO] as? Bool {
                    let noOfFrames = faceVideoRecording[FLHyperSnapConstants.NO_OF_FRAMES] as? Int
                    let fps = faceVideoRecording[FLHyperSnapConstants.FPS] as? Int
                    let bitRateM = faceVideoRecording[FLHyperSnapConstants.BIT_RATE_M] as? Double
                    setVideoRecording(shouldRecordVideo: shouldRecordVideo, numberOfFrames: noOfFrames, bitrateM: bitRateM, fps: fps)
                }
            }
        }

        if let shouldCheckActiveLiveness = (hvFaceConfig[FLHyperSnapConstants.CHECK_ACTIVE_LIVENESS] as? Bool) {
            getFaceConfig().setShouldCheckForActiveLiveness(shouldCheckActiveLiveness)
            if shouldCheckActiveLiveness {
                if let totalGestures = (hvFaceConfig[FLHyperSnapConstants.TOTAL_GESTURES] as? Int) {
                    getFaceConfig().setTotalGestures(totalGestures)
                }
            }
        }

        if let livenessApiDetails = (hvFaceConfig[FLHyperSnapConstants.LIVENESS_API_DETAILS] as? [String: Any]) {
            if !livenessApiDetails.isEmpty {
                if let livenessEndpoint = livenessApiDetails[FLHyperSnapConstants.LIVENESS_ENDPOINT] as? String {
                    setLivenessEndpoint(livenessEndpoint: livenessEndpoint )
                }
                if let livenessValue = livenessApiDetails[FLHyperSnapConstants.LIVENESS_VALUE] as? String {
                    setLivenessMode(liveness: livenessValue)
                }
                if let livenessParams = livenessApiDetails[FLHyperSnapConstants.LIVENESS_PARAMS] as? [String: Any] {
                    setLivenessAPIParameters(params: livenessParams)
                }
                if let livenessHeaders = livenessApiDetails[FLHyperSnapConstants.LIVENESS_HEADERS] as? [String: Any] {
                    setLivenessAPIHeaders(headers: livenessHeaders)
                }
            }
        }

        if let shouldReturnFullImageUrl = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_RETURN_FULL_IMAGE_URL] as? Bool) {
            getFaceConfig().setShouldReturnFullImageUri(shouldReturnFullImageUrl)
        }

        if let shouldHandleRetries = (hvFaceConfig[FLHyperSnapConstants.FACE_SHOULD_HANDLE_RETRIES] as? Bool) {
            getFaceConfig().setShouldHandleRetries(shouldHandleRetries)
        }
    }

    public func setLivenessEndpoint(livenessEndpoint: String) {
        getFaceConfig().setLivenessEndpoint(livenessEndpoint)
    }

    public func setLivenessMode(liveness: String) {
        var livenessMode: HyperSnapParams.LivenessMode?
        switch liveness {
        case "NONE":
            livenessMode = HyperSnapParams.LivenessMode.none
        case "TEXTURELIVENESS":
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        default:
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        }

        getFaceConfig().setLivenessMode(livenessMode!)
    }

    public func setLivenessAPIParameters(params: [String: Any]) {
        getFaceConfig().setLivenessAPIParameters(params as [String: AnyObject])
    }

    public func setLivenessAPIHeaders(headers: [String: Any]) {
        getFaceConfig().setLivenessAPIHeaders(headers as! [String: String])
    }

    public func setPadding(leftPadding: Double, rightPadding: Double, topPadding: Double, bottomPadding: Double) {
        getFaceConfig().setPadding(left: Float(leftPadding), right: Float(rightPadding), top: Float(topPadding), bottom: Float(bottomPadding))
    }

    public func setVideoRecording(shouldRecordVideo: Bool, numberOfFrames: Int?, bitrateM: Double?, fps: Int?) {
        getFaceConfig().setShouldRecordVideo(shouldRecordVideo)
        if shouldRecordVideo {
            if numberOfFrames != nil {
                getFaceConfig().setNumberOfFrames(numberOfFrames!)
            }
            if fps != nil {
                getFaceConfig().setFps(fps!)
            }
        }
    }

    public func setUIStrings(uiStrings: [String: Any]) {
        var uiStringsUpdated: [String: String] = [:]
        for (key, value) in uiStrings {
            if let stringValue = value as? String {
                uiStringsUpdated[key] = stringValue
            }
        }
        getFaceConfig().textConfig.setTextConfig(uiStringsUpdated)
    }

    public func start(
        context: UIViewController,
        callback: @escaping ((HVError?, HVResponse?, UIViewController) -> Void)
    ) {
        HVFaceViewController.start(
            context,
            hvFaceConfig: getFaceConfig(),
            completionHandler: callback
        )
    }
}
