import Foundation
import HyperSnapSDK

public class FLHVNetworkHelper {

    static func makeOCRCall(
        context: UIViewController,
        endpoint: String,
        documentUri: String,
        parameters: NSDictionary,
        headers: NSDictionary,
        callback: @escaping ((HVError?, HVResponse?) -> Void)
    ) {
        HVNetworkHelper.makeOCRAPICall(
            endpoint: endpoint,
            documentUri: documentUri,
            parameters: parameters as? [String: AnyObject],
            headers: headers as? [String: String],
            completionHandler: callback
        )
    }

    static func makeFaceMatchCall(
        context: UIViewController,
        endpoint: String,
        faceUri: String,
        documentUri: String,
        faceMatchModeString: String,
        parameters: NSDictionary,
        headers: NSDictionary,
        callback: @escaping ((HVError?, HVResponse?) -> Void)
    ) {
        let faceMatchMode = getFaceMatchMode(faceMatchModeString: faceMatchModeString)

        HVNetworkHelper.makeFaceMatchAPICall(
            endpoint: endpoint,
            image1: faceUri,
            image2: documentUri,
            faceMatchMode: faceMatchMode,
            parameters: parameters as? [String: AnyObject],
            headers: headers as? [String: String],
            completionHandler: callback
        )
    }

    private static func getFaceMatchMode(faceMatchModeString: String) -> HyperSnapParams.FaceMatchMode {
        switch faceMatchModeString.uppercased() {
        case "FACE_FACE":
            return .Face_Face
        case "FACE_ID_FACE_STRING":
            return .Face_IdFaceString
        default:
            return .Face_Id
        }
    }
}
