import Foundation
import HyperSnapSDK

public class FLHyperSnapSDK {

    func initialize(appId: String, appName: String, regionValue: String) {
        let region = getRegion(param: regionValue)
        HyperSnapSDKConfig.initialize(appId: appId, appKey: appName, region: region )
    }

    func initializeWithToken(appId: String, accessToken: String, regionValue: String) {
        let region = getRegion(param: regionValue)
        HyperSnapSDKConfig.initialize(appId: appId, accessToken: accessToken, region: region )
    }

    func setHyperSnapConfig(
        shouldUseSignature: Bool?,
        shouldEnableSSLPinning: Bool?,
        shouldReturnRawResponse: Bool?,
        timeout: Double?,
        json: String?
    ) {
        if shouldUseSignature != nil {
            HyperSnapSDKConfig.setShouldUseSignature(shouldUseSignature!)
        }
        if shouldEnableSSLPinning != nil {
            HyperSnapSDKConfig.setShouldEnableSSLPinning(shouldEnableSSLPinning!)
        }
        if shouldReturnRawResponse != nil {
            HyperSnapSDKConfig.setShouldReturnRawResponse(shouldReturnRawResponse!)
        }

        if timeout != nil {
            HyperSnapSDKConfig.setTimeoutIntervalForRequest(timeout!)
        }

        if json != nil {
            self.setUiConfig(json!)
        }
    }

    func setUiConfig(_ json: String) {
        let jsonDecoder = JSONDecoder()
        let configData = Data(json.utf8)
        do {
            let decodedConfig = try jsonDecoder.decode(HVUIConfig.self, from: configData)
            HyperSnapSDKConfig.setUiConfig(decodedConfig)
        } catch {
            print("Error: \(error)")
        }
    }

    //    static func setShouldUseAnalytics(_ shouldUse: Bool) {
    //        HyperSnapSDKConfig.setShouldUseAnalytics(shouldUse)
    //    }

    func sortDictionaryAlphabetically(_ dictionary: [String: AnyObject]) -> String {
        return HyperSnapSDKConfig.sortDictionaryAlphabetically(dictionary)
    }

    func startUserSession(userId: String?) -> HVSessionResponse {
        if let userId = userId {
            if userId.isEmpty {
                return HyperSnapSDKConfig.startUserSession()
            } else {
                return HyperSnapSDKConfig.startUserSession(userId)
            }
        } else {
            return HyperSnapSDKConfig.startUserSession()
        }
    }

    func endUserSession() {
        HyperSnapSDKConfig.endUserSession()
    }

    func getRegion (param: String) -> HyperSnapParams.Region {
        switch param.uppercased() {
        case "INDIA":
            return HyperSnapParams.Region.India
        case "ASIA_PACIFIC":
            return HyperSnapParams.Region.AsiaPacific
        case "UNITED_STATES":
            return HyperSnapParams.Region.UnitedStates
        case "AFRICA":
            return HyperSnapParams.Region.Africa
        default:
            return HyperSnapParams.Region.India
        }
    }
}
