import Flutter
import UIKit
import HyperSnapSDK

public class SwiftHypersnapsdkFlutterPlugin: NSObject, FlutterPlugin {
    let flHypersnapSDK = FLHyperSnapSDK()
    let flhvDocsCapture = FLHVDocsCapture()
    let flhvFaceCapture = FLHVFaceCapture()
    let flhvQRCapture = FLHVQRScanCapture()
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: FLHyperSnapConstants.METHOD_CHANNEL_STRING,
            binaryMessenger: registrar.messenger()
        )
        let instance = SwiftHypersnapsdkFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case FLHyperSnapConstants.INITIALIZE_METHOD:
            guard let args = call.arguments as? [String: Any] else {
                return result(String("iOS could not recognize flutter arguments in method: \(FLHyperSnapConstants.INITIALIZE_METHOD)"))
            }
            let appId = args[FLHyperSnapConstants.APP_ID_ARG] as? String
            let appKey = args[FLHyperSnapConstants.APP_KEY_ARG] as? String
            let region = args[FLHyperSnapConstants.REGION_ARG] as? String
            flHypersnapSDK.initialize(appId: appId!, appName: appKey!, regionValue: region!)
            result(String("Initialised"))
            
        case FLHyperSnapConstants.INITIALIZE_WITH_TOKEN_METHOD:
            guard let args = call.arguments as? [String: Any] else {
                return result(String("iOS could not recognize flutter arguments in method: \(FLHyperSnapConstants.INITIALIZE_WITH_TOKEN_METHOD)"))
            }
            let appId = args[FLHyperSnapConstants.APP_ID_ARG] as? String
            let accessToken = args[FLHyperSnapConstants.ACCESS_TOKEN_ARG] as? String
            let region = args[FLHyperSnapConstants.REGION_ARG] as? String
            flHypersnapSDK.initializeWithToken(appId: appId!, accessToken: accessToken!, regionValue: region!)
            result(String("Initialised"))
            
        case FLHyperSnapConstants.HYPER_SNAP_SDK_CONFIG_METHOD:
            guard let args = call.arguments as? [String: Any] else {
                return result(String("iOS could not recognize flutter arguments in method: \(FLHyperSnapConstants.HYPER_SNAP_SDK_CONFIG_METHOD)"))
            }
            let shouldEnableSSLPinning = args[FLHyperSnapConstants.SHOULD_ENABLE_SSL_PINNING_ARG] as? Bool
            let shouldUseSignature = args[FLHyperSnapConstants.SHOULD_USE_SIGNATURE_ARG] as? Bool
            let shouldReturnRawResponse = args[FLHyperSnapConstants.SHOULD_RETURN_RAW_RESPONSE_ARG] as? Bool
            let readTimeout = args[FLHyperSnapConstants.READ_TIMEOUT_ARG] as? Double
            let json = args[FLHyperSnapConstants.UI_CONFIG_MAP] as? String
            flHypersnapSDK.setHyperSnapConfig(
                shouldUseSignature: shouldUseSignature,
                shouldEnableSSLPinning: shouldEnableSSLPinning,
                shouldReturnRawResponse: shouldReturnRawResponse,
                timeout: readTimeout,
                json: json
            )
            result("sdk config set")
            
        case FLHyperSnapConstants.START_USER_SESSION_METHOD:
            guard let args = call.arguments as? [String: Any] else {
                return result(String("iOS could not recognize flutter arguments in method: \(FLHyperSnapConstants.START_USER_SESSION_METHOD)"))
            }
            let userId = args[FLHyperSnapConstants.USER_ID_ARG] as? String
            let hvSessionResponse = flHypersnapSDK.startUserSession(userId: userId)
            if hvSessionResponse.isSuccess() {
                result(true)
            } else {
                print("Error code: \(hvSessionResponse.getHVError()!.errorCode), Error Message: \(hvSessionResponse.getHVError()!.errorMessage)")
                result(false)
            }
            
        case FLHyperSnapConstants.END_USER_SESSION_METHOD:
            flHypersnapSDK.endUserSession()
            result("User session ended")
            
        case FLHyperSnapConstants.SORT_JSON_KEYS_ALPHABETICALLY_METHOD:
            guard let args = call.arguments as? [String: Any] else {
                return result(String("iOS could not recognize flutter arguments in method: \(FLHyperSnapConstants.SORT_JSON_KEYS_ALPHABETICALLY_METHOD)"))
            }
            let map = args["map"] as! [String: AnyObject]
            result(flHypersnapSDK.sortDictionaryAlphabetically(map))
            
        case FLHyperSnapConstants.START_DOC_CAPTURE:
            if let args = call.arguments as? [String: Any] {
                let docConfig = args[FLHyperSnapConstants.HV_DOC_CONFIG] as! [String: AnyObject]
                flhvDocsCapture.createNewDocConfig()
                flhvDocsCapture.setHVDocConfig(hvDocConfig: docConfig)
            }
            
            let completionHandler: (_ error: HVError?, _ response: HVResponse?, _ viewController: UIViewController) -> Void = { error, hvResult, _ in
                let map = [
                    "errorObj": self.getErrorObj(error),
                    "resultObj": self.getResultObj(hvResult)
                ]
                result(map)
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController {
                flhvDocsCapture.start(
                    context: topController,
                    callback: completionHandler
                )
            }
            
        case FLHyperSnapConstants.START_FACE_CAPTURE:
            if let args = call.arguments as? [String: Any] {
                let faceConfig = args[FLHyperSnapConstants.HV_FACE_CONFIG] as! [String: AnyObject]
                flhvFaceCapture.createNewFaceConfig()
                flhvFaceCapture.setHVFaceConfig(hvFaceConfig: faceConfig)
            }
            
            let completionHandler: (_ error: HVError?, _ response: HVResponse?, _ viewController: UIViewController) -> Void = { error, hvResult, _ in
                let map = [
                    "errorObj": self.getErrorObj(error),
                    "resultObj": self.getResultObj(hvResult)
                ]
                result(map)
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController {
                flhvFaceCapture.start(
                    context: topController,
                    callback: completionHandler
                )
            }
            
        case FLHyperSnapConstants.NETWORK_HELPER_MAKE_OCR_CALL:
            guard let args = call.arguments as? [String: Any] else {
                // result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                return
            }
            let endpoint = args[FLHyperSnapConstants.NETWORK_HELPER_OCR_ENDPOINT] as? String
            let documentUri = args[FLHyperSnapConstants.NETWORK_HELPER_OCR_DOCUMENT_URI] as? String
            let parameters = args[FLHyperSnapConstants.NETWORK_HELPER_OCR_PARAMETERS] as? NSDictionary
            let headers = args[FLHyperSnapConstants.NETWORK_HELPER_OCR_HEADERS] as? NSDictionary
            let completionHandler: (_ error: HVError?, _ response: HVResponse?) -> Void = {error, hvResult in
                let map = [
                    "errorObj": self.getErrorObj(error),
                    "resultObj": self.getResultObj(hvResult)
                ]
                result(map)
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController {
                FLHVNetworkHelper.makeOCRCall(
                    context: topController,
                    endpoint: endpoint!,
                    documentUri: documentUri!,
                    parameters: parameters!,
                    headers: headers!,
                    callback: completionHandler
                )
            }
            
        case FLHyperSnapConstants.NETWORK_HELPER_MAKE_FACE_MATCH_CALL:
            guard let args = call.arguments as? [String: Any] else {
                // result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                return
            }
            let endpoint = args[FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_ENDPOINT] as? String
            let documentUri = args[FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_DOCUMENT_URI] as? String
            let faceUri = args[FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_FACE_URI] as? String
            let faceMatchMode = args[FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_FACE_MATCH_MODE] as? String
            let parameters = args[FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_PARAMETERS] as? NSDictionary
            let headers = args[FLHyperSnapConstants.NETWORK_HELPER_FACE_MATCH_HEADERS] as? NSDictionary
            let completionHandler: (_ error: HVError?, _ response: HVResponse?) -> Void = { error, hvResult in
                let map = [
                    "errorObj": self.getErrorObj(error),
                    "resultObj": self.getResultObj(hvResult)
                ]
                result(map)
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController {
                FLHVNetworkHelper.makeFaceMatchCall(
                    context: topController,
                    endpoint: endpoint!,
                    faceUri: faceUri!,
                    documentUri: documentUri!,
                    faceMatchModeString: faceMatchMode!,
                    parameters: parameters!,
                    headers: headers!,
                    callback: completionHandler
                )
            }
            
        case FLHyperSnapConstants.QR_SCAN_CAPTURE_START:
            if let args = call.arguments as? [String: Any] {
                let qrConfig = args[FLHyperSnapConstants.HV_QR_CONFIG] as! [String: AnyObject]
                flhvQRCapture.createNewQRConfig()
                flhvQRCapture.setHVQRConfig(qrConfig: qrConfig)
            }
            
            let completionHandler: (_ error: HVError?, _ result: [String: AnyObject]?) -> Void = { hvError, hvResult in
                var errorObj = [String: Any]()
                var resultObj = [String: AnyObject]()
                resultObj["qr"] = "" as AnyObject
                if let error = hvError {
                    // Error Code: error.errorCode, Error Message: error.errorMessage
                    errorObj["errorCode"] = error.errorCode
                    errorObj["errorMessage"] = error.errorMessage
                }
                if let result = hvResult {
                    resultObj = result
                }
                
                let map = [
                    "errorObj": errorObj,
                    "resultObj": resultObj
                ]
                result(map)
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController {
                flhvQRCapture.start(
                    context: topController,
                    callback: completionHandler
                )
            }
            
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    func getErrorObj(_ hvError: HVError?) -> [String: Any] {
        var errorObj = [String: Any]()
        if hvError != nil {
            // Error Code: error.errorCode, Error Message: error.errorMessage
            errorObj["errorCode"] = hvError?.errorCode
            errorObj["errorMessage"] = hvError?.errorMessage
        }
        return errorObj
    }
    
    func getResultObj(_ hvResult: HVResponse?) -> [String: Any] {
        var resultObj = [String: Any]()
        if let result = hvResult {
            do {
                if let imageUri = result.imageUri,
                   !imageUri.isEmpty {
                    resultObj["imageUri"] = imageUri
                }
                
                if let fullImageUri = result.fullImageUri,
                   !fullImageUri.isEmpty {
                    resultObj["fullImageUri"] = fullImageUri
                }
                
                if let retakeMessage = result.retakeMessage,
                   !retakeMessage.isEmpty {
                    resultObj["retakeMessage"] = retakeMessage
                }
                
                if let action = result.action,
                   !action.isEmpty {
                    resultObj["action"] = action
                }
                
                if let videoUri = result.videoUri,
                   !videoUri.isEmpty {
                    resultObj["videoUri"] = videoUri
                }
                
                if !result.gestureLivenessImageUrls.isEmpty {
                    resultObj["gestureLivenessUrls"] = result.gestureLivenessImageUrls
                }
                
                if let rawBarcode = result.rawBarcode {
                    resultObj["rawBarcode"] = rawBarcode
                }
                
                if !result.retakeAttemptResponses.isEmpty {
                    resultObj["retakeAttemptResponses"] = self.getFailedResponse(result)
                }
                
                if let response = result.apiResult {
                    let jsonData = try JSONSerialization.data(withJSONObject: response, options: JSONSerialization.WritingOptions.prettyPrinted)
                    let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                    resultObj["apiResult"] = jsonString
                } else {
                    resultObj["apiResult"] = "{}"
                }
                
                if let headers = result.apiHeaders {
                    let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
                    let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                    resultObj["apiHeaders"] = headersString
                    
                } else {
                    resultObj["apiHeaders"] = "{}"
                }
            } catch let myJSONError {
                print("getResultObj : \(myJSONError.localizedDescription)")
            }
        }
        return resultObj
    }
    
    func getFailedResponse(_ result: HVResponse) -> NSMutableArray {
        let responses = result.retakeAttemptResponses
        let retakeAttempts = NSMutableArray()
        for response in responses {
            var resultDict = NSMutableDictionary()
            resultDict = getHVResponseDict(result: response, resultDict: resultDict)
            retakeAttempts.add(resultDict)
        }
        return retakeAttempts
    }
    
    func getHVResponseDict(result: HVBaseResponse, resultDict: NSMutableDictionary ) -> NSMutableDictionary {
        do {
            if let imageUri = result.imageUri {
                if !imageUri.isEmpty {
                    resultDict["imageUri"] = imageUri
                }
            }
            
            if let fullImageUri = result.fullImageUri {
                if !fullImageUri.isEmpty {
                    resultDict["fullImageUri"] = fullImageUri
                }
            }
            
            if let retakeMessage = result.retakeMessage {
                if !retakeMessage.isEmpty {
                    resultDict["retakeMessage"] = retakeMessage
                }
            }
            
            if let action = result.action {
                if !action.isEmpty {
                    resultDict["action"] = action
                }
            }
            
            if let videoUri = result.videoUri {
                if !videoUri.isEmpty {
                    resultDict["videoUri"] = videoUri
                }
            }
            
            let gestureLivenessUrls = result.gestureLivenessImageUrls
            if !gestureLivenessUrls.isEmpty {
                resultDict["gestureLivenessUrls"] = gestureLivenessUrls
            }
            
            if let rawBarcode = result.rawBarcode {
                resultDict["rawBarcode"] = rawBarcode
            }
            
            if let response = result.apiResult {
                let jsonData = try JSONSerialization.data(withJSONObject: response, options: JSONSerialization.WritingOptions.prettyPrinted)
                let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                resultDict["apiResult"] = jsonString
            } else {
                resultDict["apiResult"] = "{}"
            }
            
            if let headers = result.apiHeaders {
                let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
                let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                resultDict["apiHeaders"] = headersString
            } else {
                resultDict["apiHeaders"] = "{}"
            }
        } catch let myJSONError {
            print(myJSONError)
        }
        
        return resultDict
    }
}
