import 'dart:convert';

import 'hyper_snap_sdk_constants.dart';

class HVUIConfig {
  HVUIColor? hvUIColor;
  HVUIFont? hvUIFont;
  HVUIFontSize? hvUIFontSize;
  HVUIFontWeight? hvUIFontWeight;
  HVUIAnimation? hvUIAnimation;
  HVUIBorderRadius? hvUIBorderRadius;
  HVUIAlignment? hvUIAlignment;
  Map? _uiConfigMap;

  ///HVUIConfig() - used to config all kinds of UI for all hypersnap SDK screens
  HVUIConfig(
      {this.hvUIAlignment,
      this.hvUIAnimation,
      this.hvUIColor,
      this.hvUIBorderRadius,
      this.hvUIFont,
      this.hvUIFontSize,
      this.hvUIFontWeight});

  static HVUIConfig fromMap(Map uiConfigMap) {
    var hvUIConfig = HVUIConfig();
    hvUIConfig._uiConfigMap = uiConfigMap;
    return hvUIConfig;
  }

  /// Convert HVUIConfig object to Map
  String toJSONString() {
    if (_uiConfigMap != null) {
      return json.encode(_uiConfigMap);
    }
    return json.encode({
      HyperSnapSDKConstants.hvUIColor: hvUIColor?.toMap(),
      HyperSnapSDKConstants.hvUIFont: hvUIFont?.toMap(),
      HyperSnapSDKConstants.hvUIFontWeight: hvUIFontWeight?.toMap(),
      HyperSnapSDKConstants.hvUIFontSize: hvUIFontSize?.toMap(),
      HyperSnapSDKConstants.hvUIAnimation: hvUIAnimation?.toMap(),
      HyperSnapSDKConstants.hvUIBorderRadius: hvUIBorderRadius?.toMap(),
      HyperSnapSDKConstants.hvUIAlignment: hvUIAlignment?.toMap(),
    });
  }
}

class HVUIColor {
  String? titleTextColor;
  String? descriptionTextColor;
  String? documentSideHintTextColor;
  String? statusTextColor;
  String? primaryButtonBackgroundColor;
  String? primaryButtonTextColor;
  String? primaryButtonBorderColor;
  String? secondaryButtonBackgroundColor;
  String? secondaryButtonTextColor;
  String? secondaryButtonBorderColor;
  String? captureButtonColor;
  String? retakeMessageColor;
  String? alertTextBoxTextColor;
  String? alertTextBoxBorderColor;
  String? alertTextBoxBackgroundColor;

  HVUIColor(
      {this.alertTextBoxBackgroundColor,
      this.alertTextBoxBorderColor,
      this.alertTextBoxTextColor,
      this.captureButtonColor,
      this.descriptionTextColor,
      this.documentSideHintTextColor,
      this.primaryButtonBackgroundColor,
      this.primaryButtonBorderColor,
      this.primaryButtonTextColor,
      this.retakeMessageColor,
      this.secondaryButtonBackgroundColor,
      this.secondaryButtonBorderColor,
      this.secondaryButtonTextColor,
      this.statusTextColor,
      this.titleTextColor});

  /// Convert HVUIColor object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextColor: titleTextColor,
      HyperSnapSDKConstants.descriptionTextColor: descriptionTextColor,
      HyperSnapSDKConstants.documentSideHintTextColor:
          documentSideHintTextColor,
      HyperSnapSDKConstants.statusTextColor: statusTextColor,
      HyperSnapSDKConstants.primaryButtonBackgroundColor:
          primaryButtonBackgroundColor,
      HyperSnapSDKConstants.primaryButtonTextColor: primaryButtonTextColor,
      HyperSnapSDKConstants.primaryButtonBorderColor: primaryButtonBorderColor,
      HyperSnapSDKConstants.secondaryButtonBackgroundColor:
          secondaryButtonBackgroundColor,
      HyperSnapSDKConstants.secondaryButtonTextColor: secondaryButtonTextColor,
      HyperSnapSDKConstants.secondaryButtonBorderColor:
          secondaryButtonBorderColor,
      HyperSnapSDKConstants.captureButtonColor: captureButtonColor,
      HyperSnapSDKConstants.retakeMessageColor: retakeMessageColor,
      HyperSnapSDKConstants.alertTextBoxTextColor: alertTextBoxTextColor,
      HyperSnapSDKConstants.alertTextBoxBorderColor: alertTextBoxBorderColor,
      HyperSnapSDKConstants.alertTextBoxBackgroundColor:
          alertTextBoxBackgroundColor,
    };
  }
}

class HVUIFont {
  String? titleTextFont;
  String? descriptionTextFont;
  String? statusTextFont;
  String? documentSideHintTextFont;
  String? retakeMessageFont;
  String? primaryButtonTextFont;
  String? secondaryButtonTextFont;
  String? alertTextBoxTextFont;

  HVUIFont(
      {this.alertTextBoxTextFont,
      this.descriptionTextFont,
      this.documentSideHintTextFont,
      this.primaryButtonTextFont,
      this.retakeMessageFont,
      this.secondaryButtonTextFont,
      this.statusTextFont,
      this.titleTextFont});

  /// Convert HVUIFont object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextFont: titleTextFont,
      HyperSnapSDKConstants.descriptionTextFont: descriptionTextFont,
      HyperSnapSDKConstants.statusTextFont: statusTextFont,
      HyperSnapSDKConstants.documentSideHintTextFont: documentSideHintTextFont,
      HyperSnapSDKConstants.retakeMessageFont: retakeMessageFont,
      HyperSnapSDKConstants.primaryButtonTextFont: primaryButtonTextFont,
      HyperSnapSDKConstants.secondaryButtonTextFont: secondaryButtonTextFont,
      HyperSnapSDKConstants.alertTextBoxTextFont: alertTextBoxTextFont,
    };
  }
}

class HVUIFontWeight {
  String? titleTextWeight;
  String? descriptionTextWeight;
  String? statusTextWeight;
  String? documentSideHintTextWeight;
  String? retakeMessageWeight;
  String? primaryButtonTextWeight;
  String? secondaryButtonTextWeight;
  String? alertTextBoxTextWeight;

  HVUIFontWeight(
      {this.alertTextBoxTextWeight,
      this.descriptionTextWeight,
      this.documentSideHintTextWeight,
      this.primaryButtonTextWeight,
      this.retakeMessageWeight,
      this.secondaryButtonTextWeight,
      this.statusTextWeight,
      this.titleTextWeight});

  /// Convert HVUIFontWeight object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextWeight: titleTextWeight,
      HyperSnapSDKConstants.descriptionTextWeight: descriptionTextWeight,
      HyperSnapSDKConstants.statusTextWeight: statusTextWeight,
      HyperSnapSDKConstants.documentSideHintTextWeight:
          documentSideHintTextWeight,
      HyperSnapSDKConstants.retakeMessageWeight: retakeMessageWeight,
      HyperSnapSDKConstants.primaryButtonTextWeight: primaryButtonTextWeight,
      HyperSnapSDKConstants.secondaryButtonTextWeight:
          secondaryButtonTextWeight,
      HyperSnapSDKConstants.alertTextBoxTextWeight: alertTextBoxTextWeight,
    };
  }
}

class HVUIFontSize {
  double? titleTextSize;
  double? descriptionTextSize;
  double? statusTextSize;
  double? documentSideHintTextSize;
  double? retakeMessageTextSize;
  double? primaryButtonTextSize;
  double? secondaryButtonTextSize;
  double? alertTextBoxTextSize;

  HVUIFontSize(
      {this.alertTextBoxTextSize,
      this.descriptionTextSize,
      this.documentSideHintTextSize,
      this.primaryButtonTextSize,
      this.retakeMessageTextSize,
      this.secondaryButtonTextSize,
      this.statusTextSize,
      this.titleTextSize});

  /// Convert HVUIFontSize object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextSize: titleTextSize,
      HyperSnapSDKConstants.descriptionTextSize: descriptionTextSize,
      HyperSnapSDKConstants.statusTextSize: statusTextSize,
      HyperSnapSDKConstants.documentSideHintTextSize: documentSideHintTextSize,
      HyperSnapSDKConstants.retakeMessageTextSize: retakeMessageTextSize,
      HyperSnapSDKConstants.primaryButtonTextSize: primaryButtonTextSize,
      HyperSnapSDKConstants.secondaryButtonTextSize: secondaryButtonTextSize,
      HyperSnapSDKConstants.alertTextBoxTextSize: alertTextBoxTextSize,
    };
  }
}

class HVUIAnimation {
  String? documentInstructionLottie;
  String? documentInstructionFrontSideLottie;
  String? documentInstructionBackSideLottie;
  String? documentProcessingLottie;
  String? documentSuccessLottie;
  String? documentFailureLottie;
  String? faceInstructionLottie;
  String? faceProcessingLottie;
  String? faceSuccessLottie;
  String? faceFailureLottie;
  String? endStateProcessing;
  String? endStateSuccess;
  String? endStateFailure;
  String? uploadFailure;
  String? qrInstruction;

  HVUIAnimation(
      {this.documentFailureLottie,
      this.documentInstructionBackSideLottie,
      this.documentInstructionFrontSideLottie,
      this.documentInstructionLottie,
      this.documentProcessingLottie,
      this.documentSuccessLottie,
      this.endStateFailure,
      this.endStateProcessing,
      this.endStateSuccess,
      this.faceFailureLottie,
      this.faceInstructionLottie,
      this.faceProcessingLottie,
      this.faceSuccessLottie,
      this.qrInstruction,
      this.uploadFailure});

  /// Convert HVUIAnimation object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.documentInstructionLottie:
          documentInstructionLottie,
      HyperSnapSDKConstants.documentInstructionFrontSideLottie:
          documentInstructionFrontSideLottie,
      HyperSnapSDKConstants.documentInstructionBackSideLottie:
          documentInstructionBackSideLottie,
      HyperSnapSDKConstants.documentProcessingLottie: documentProcessingLottie,
      HyperSnapSDKConstants.documentSuccessLottie: documentSuccessLottie,
      HyperSnapSDKConstants.documentFailureLottie: documentFailureLottie,
      HyperSnapSDKConstants.faceInstructionLottie: faceInstructionLottie,
      HyperSnapSDKConstants.faceProcessingLottie: faceProcessingLottie,
      HyperSnapSDKConstants.faceSuccessLottie: faceSuccessLottie,
      HyperSnapSDKConstants.faceFailureLottie: faceFailureLottie,
      HyperSnapSDKConstants.endStateProcessing: endStateProcessing,
      HyperSnapSDKConstants.endStateSuccess: endStateSuccess,
      HyperSnapSDKConstants.endStateFailure: endStateFailure,
      HyperSnapSDKConstants.uploadFailure: uploadFailure,
      HyperSnapSDKConstants.qrInstruction: qrInstruction,
    };
  }
}

class HVUIBorderRadius {
  double? primaryButtonRadius;
  double? secondaryButtonRadius;

  HVUIBorderRadius({this.primaryButtonRadius, this.secondaryButtonRadius});

  /// Convert HVUIButtonRadius object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.primaryButtonRadius: primaryButtonRadius,
      HyperSnapSDKConstants.secondaryButtonRadius: secondaryButtonRadius,
    };
  }
}

class HVUIAlignment {
  String? titleTextAlignment;
  String? descriptionTextAlignment;
  String? statusTextAlignment;

  HVUIAlignment(
      {this.descriptionTextAlignment,
      this.statusTextAlignment,
      this.titleTextAlignment});

  /// Convert HVUIAlignment object to Map
  Map<String, dynamic> toMap() {
    return {
      HyperSnapSDKConstants.titleTextAlignment: titleTextAlignment,
      HyperSnapSDKConstants.descriptionTextAlignment: descriptionTextAlignment,
      HyperSnapSDKConstants.statusTextAlignment: statusTextAlignment,
    };
  }
}
